<?php

/**
 * @since      1.0.0
 * @package    Woo_PayPal_Pro
 * @subpackage Woo_PayPal_Pro/includes
 * @author     easypayment <wpeasypayment@gmail.com>
 */
class Woo_PayPal_Pro {

    protected $loader;
    protected $plugin_name;
    protected $version;
    public function __construct() {
        $this->plugin_name = 'woo-paypal-pro';
        $this->version = '7.0.0';
        $this->load_dependencies();
        $this->set_locale();
        $this->woo_gateway_hooks();
        add_action('parse_request', array($this, 'handle_api_requests'), 0);
        add_action('woo_paypal_pro_api_ipn_handler', array($this, 'easy_woo_paypal_pro_api_ipn_handler'));
        $prefix = is_network_admin() ? 'network_admin_' : '';
        add_filter("{$prefix}plugin_action_links_" . PAYPAL_PRO_PLUGIN_BASENAME, array($this, 'paypal_pro_plugin_action_links'), 10, 4);
    }

    private function load_dependencies() {
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-woo-paypal-pro-loader.php';
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-woo-paypal-pro-i18n.php';
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-woo-paypal-pro-common-function.php';
        if (class_exists('WC_Payment_Gateway')) {
            require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-woo-paypal-pro-gateway.php';
        }
        $this->loader = new Woo_PayPal_Pro_Loader();
    }

    private function set_locale() {
        $plugin_i18n = new Woo_PayPal_Pro_i18n();
        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
    }

    private function woo_gateway_hooks() {
        add_filter('woocommerce_payment_gateways', array($this, 'easy_methods_woo_paypal_pro_gateways'), 10, 1);
    }

    public function handle_api_requests() {
        global $wp;
        if (isset($_GET['action']) && $_GET['action'] == 'ipn_handler') {
            $wp->query_vars['Woo_PayPal_Pro'] = wc_clean($_GET['action']);
        }
        if (!empty($wp->query_vars['Woo_PayPal_Pro'])) {
            ob_start();
            $api = strtolower(esc_attr($wp->query_vars['Woo_PayPal_Pro']));
            do_action('woo_paypal_pro_api_' . strtolower($api));
            ob_end_clean();
            die('1');
        }
    }

    public function run() {
        $this->loader->run();
    }

    public function get_plugin_name() {
        return $this->plugin_name;
    }

    public function get_loader() {
        return $this->loader;
    }

    public function get_version() {
        return $this->version;
    }

    public function easy_methods_woo_paypal_pro_gateways($methods) {
        $methods[] = 'Easy_Woo_PayPal_Pro_Gateway';
        return $methods;
    }

    public function easy_woo_paypal_pro_api_ipn_handler() {
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-woo-paypal-pro-paypal-listner.php';
        $Pal_Pro_PayPal_listner = new Easy_PayPal_IPN_Listener();
        $Pal_Pro_PayPal_listner->check_response();
    }

    public function paypal_pro_plugin_action_links($actions, $plugin_file, $plugin_data, $context) {
        $custom_actions = array(
            'configure' => sprintf('<a href="%s">%s</a>', admin_url('admin.php?page=wc-settings&tab=checkout&section=paypal_pro'), __('Configure', 'donation-button')),
            'docs' => sprintf('<a href="%s" target="_blank">%s</a>', 'https://www.easy.com/product/paypal-pro-for-woocommerce/', __('Docs', 'donation-button')),
            'support' => sprintf('<a href="%s" target="_blank">%s</a>', 'https://wordpress.org/support/plugin/woo-paypal-pro', __('Support', 'donation-button')),
            'review' => sprintf('<a href="%s" target="_blank">%s</a>', 'https://wordpress.org/support/view/plugin-reviews/woo-paypal-pro', __('Write a Review', 'donation-button')),
        );
        return array_merge($custom_actions, $actions);
    }
}
