<?php

/**
 * @wordpress-plugin
 * Plugin Name:       Payment Gateway for PayPal Pro on WooCommerce
 * Plugin URI:        https://profiles.wordpress.org/easypayment
 * Description:       PayPal Pro works by adding credit card fields on the checkout and then sending the details to PayPal for verification
 * Version:           7.0.0
 * Author:            easypayment
 * Author URI:        https://profiles.wordpress.org/easypayment/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       woo-paypal-pro
 * Domain Path:       /languages
 * Requires at least: 3.8
 * Tested up to: 6.7.2
 * WC requires at least: 3.0.0
 * WC tested up to: 9.6.1
 */
// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

if (!defined('WOO_PAYPAL_PRO_PLUGIN_DIR')) {
    define('WOO_PAYPAL_PRO_PLUGIN_DIR', dirname(__FILE__));
}

if (!defined('WOO_PAYPAL_PRO_PLUGIN_VERSION')) {
    define('WOO_PAYPAL_PRO_PLUGIN_VERSION', '7.0.0');
}

if (!defined('PAYPAL_PRO_PLUGIN_BASENAME')) {
    define('PAYPAL_PRO_PLUGIN_BASENAME', plugin_basename(__FILE__));
}
if (!defined('PAYPAL_PRO_PLUGIN_ASSET_URL')) {
    define('PAYPAL_PRO_PLUGIN_ASSET_URL', plugin_dir_url(__FILE__));
}
function activate_woo_paypal_pro() {
    require_once plugin_dir_path(__FILE__) . 'includes/class-woo-paypal-pro-activator.php';
    Woo_PayPal_Pro_Activator::activate();
}

function deactivate_woo_paypal_pro() {
    require_once plugin_dir_path(__FILE__) . 'includes/class-woo-paypal-pro-deactivator.php';
    Woo_PayPal_Pro_Deactivator::deactivate();
}

register_activation_hook(__FILE__, 'activate_woo_paypal_pro');
register_deactivation_hook(__FILE__, 'deactivate_woo_paypal_pro');

require plugin_dir_path(__FILE__) . 'includes/class-woo-paypal-pro.php';

function run_woo_paypal_pro() {

    $plugin = new Woo_PayPal_Pro();
    $plugin->run();
}

add_action('plugins_loaded', 'load_woo_paypal_pro');

function load_woo_paypal_pro() {
    run_woo_paypal_pro();
}

add_action('before_woocommerce_init', function () {
    if (class_exists(\Automattic\WooCommerce\Utilities\FeaturesUtil::class)) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('cart_checkout_blocks', __FILE__, true);
    }
});
add_action('woocommerce_blocks_loaded', function () {
    try {
        if (!class_exists('Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType')) {
            return;
        }
        require_once( WOO_PAYPAL_PRO_PLUGIN_DIR . '/checkout-block/ppcp-pro-block.php' );
        add_action(
                'woocommerce_blocks_payment_method_type_registration',
                function (Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry) {
                    $payment_method_registry->register(new Woo_PayPal_Pro_Block);
                }
        );
    } catch (Exception $ex) {
        
    }
});